@@ -86,8 +86,11 @@ void add_line_buffer(struct session *ses, char *line, int more_output)
 	int lines;
 	int sav_row, sav_col, cur_row, cur_col, top_row, bot_row;
 
+	push_call("add_line_buffer(%p,%s,%d)",ses,line,more_output);
+
 	if (ses->buffer == NULL || HAS_BIT(ses->flags, SES_FLAG_SCROLLSTOP))
 	{
+		pop_call();
 		return;
 	}
 
@@ -104,6 +107,7 @@ void add_line_buffer(struct session *ses, char *line, int more_output)
 		{
 			strcat(ses->more_output, line);
 
+			pop_call();
 			return;
 		}
 	}
@@ -193,6 +197,7 @@ void add_line_buffer(struct session *ses, char *line, int more_output)
 	ses->top_row = top_row;
 	ses->bot_row = bot_row;
 
+	pop_call();
 	return;
 }
 
