@@ -55,13 +55,13 @@ DO_COMMAND(do_chat)
 
 	if (*cmd == 0)
 	{
-		tintin_header(NULL, " CHAT COMMANDS ");
+		tintin_header(ses, " CHAT COMMANDS ");
 
 		for (cnt = 0 ; *chat_table[cnt].name != 0 ; cnt++)
 		{
 			tintin_printf2(ses, "  [%-13s] %s", chat_table[cnt].name, chat_table[cnt].desc);
 		}
-		tintin_header(NULL, "");
+		tintin_header(ses, "");
 
 		return ses;
 	}
@@ -81,8 +81,9 @@ DO_COMMAND(do_chat)
 		}
 
 		arg = get_arg_in_braces(arg, left,  chat_table[cnt].lval);
+		substitute(ses, left, left, SUB_VAR|SUB_FUN);
 		arg = get_arg_in_braces(arg, right, chat_table[cnt].rval);
-
+		substitute(ses, right, right, SUB_VAR|SUB_FUN);
 		chat_table[cnt].fun(left, right);
 
 		return ses;
@@ -105,7 +106,7 @@ DO_CHAT(chat_initialize)
 	struct hostent *hp = NULL;
 	struct linger ld;
 	char *reuse = "1";
-	int sock;
+	int sock, port;
 
 	if (gtd->chat)
 	{
@@ -114,15 +115,7 @@ DO_CHAT(chat_initialize)
 		return;
 	}
 
-	gtd->chat = calloc(1, sizeof(struct chat_data));
-
-	gtd->chat->port     = *left ? atoi(left) : DEFAULT_PORT;
-	gtd->chat->name     = strdup("TinTin");
-	gtd->chat->ip       = strdup("<Unknown>");
-	gtd->chat->reply    = strdup("");
-	gtd->chat->color    = strdup("\033[0;1;31m");
-
-	chat_downloaddir("", "");
+	port = atoi(left) ? atoi(left) : DEFAULT_PORT;
 
 	gethostname(hostname, BUFFER_SIZE);
 
@@ -136,7 +129,7 @@ DO_CHAT(chat_initialize)
 	}
 
 	sa.sin_family      = hp->h_addrtype;
-	sa.sin_port        = htons(gtd->chat->port);
+	sa.sin_port        = htons(port);
 	sa.sin_addr.s_addr = 0;
 
 	sock = socket(AF_INET, SOCK_STREAM, 0);
@@ -166,7 +159,7 @@ DO_CHAT(chat_initialize)
 
 	if (bind(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0)
 	{
-		chat_printf("Port in use, cannot initiate chat.");
+		tintin_printf(NULL, "Port %d is already in use, cannot initiate chat.", port);
 
 		close(sock);
 
@@ -181,7 +174,16 @@ DO_CHAT(chat_initialize)
 
 		return;
 	}
-	gtd->chat->fd = sock;
+	gtd->chat = (struct chat_data *) calloc(1, sizeof(struct chat_data));
+
+	gtd->chat->fd       = sock;
+	gtd->chat->port     = port;
+
+	gtd->chat->color    = strdup("\033[0;1;31m");
+	gtd->chat->download = strdup("");
+	gtd->chat->ip       = strdup("<Unknown>");
+	gtd->chat->name     = strdup("TinTin");
+	gtd->chat->reply    = strdup("");
 
 	chat_printf("Initialized chat on port %d.", gtd->chat->port);
 }
@@ -211,7 +213,7 @@ int chat_new(int s)
 {
 	struct chat_data *new_buddy;
 	struct sockaddr_in sock;
-	unsigned int i;
+	socklen_t i;
 	int fd;
 
 	i = sizeof(sock);
@@ -237,12 +239,15 @@ int chat_new(int s)
 		return -1;
 	}
 
-	new_buddy = calloc(1, sizeof(struct chat_data));
+	new_buddy = (struct chat_data *) calloc(1, sizeof(struct chat_data));
 
-	new_buddy->ip    = strdup(inet_ntoa(sock.sin_addr));
-	new_buddy->fd    = fd;
-	new_buddy->name  = strdup("Unknown");
-	new_buddy->group = strdup("");
+	new_buddy->fd       = fd;
+
+	new_buddy->download = strdup("");
+	new_buddy->group    = strdup("");
+	new_buddy->ip       = strdup(inet_ntoa(sock.sin_addr));
+	new_buddy->name     = strdup("Unknown");
+	new_buddy->version  = strdup("");
 
 	new_buddy->timeout = CALL_TIMEOUT + time(NULL);
 
@@ -355,11 +360,14 @@ void *threaded_chat_call(void *arg)
 
 	new_buddy = calloc(1, sizeof(struct chat_data));
 
-	new_buddy->ip    = strdup(host);
-	new_buddy->port  = atoi(port);
-	new_buddy->fd    = sock;
-	new_buddy->name  = strdup("");
-	new_buddy->group = strdup("");
+	new_buddy->fd       = sock;
+	new_buddy->port     = atoi(port);
+
+	new_buddy->group    = strdup("");
+	new_buddy->ip       = strdup(host);
+	new_buddy->name     = strdup("");
+	new_buddy->version  = strdup("");
+	new_buddy->download = strdup("");
 
 	strip_vt102_codes(gtd->chat->name, name);
 
@@ -509,13 +517,16 @@ void *threaded_chat_call(void *arg)
 		return NULL;
 	}
 
-	new_buddy = calloc(1, sizeof(struct chat_data));
+	new_buddy = (struct chat_data *) calloc(1, sizeof(struct chat_data));
 
-	new_buddy->ip    = strdup(host);
-	new_buddy->port  = atoi(port);
-	new_buddy->fd    = sock;
-	new_buddy->name  = strdup("");
-	new_buddy->group = strdup("");
+	new_buddy->fd       = sock;
+	new_buddy->port     = atoi(port);
+
+	new_buddy->download = strdup("");
+	new_buddy->group    = strdup("");
+	new_buddy->ip       = strdup(host);
+	new_buddy->name     = strdup("");
+	new_buddy->version  = strdup("");
 
 	strip_vt102_codes(gtd->chat->name, name);
 
@@ -574,7 +585,7 @@ DO_CHAT(chat_call)
 
 	sprintf(buf, "{%s} {%s}", left, right);
 
-	pthread_create(&thread, NULL, (void *) threaded_chat_call, (void *) buf);
+	pthread_create(&thread, NULL, threaded_chat_call, (void *) buf);
 }
 	
 #else
@@ -615,14 +626,14 @@ void close_chat(struct chat_data *buddy, int unlink)
 		}
 	}
 
-	STRFREE(buddy->name);
-	STRFREE(buddy->ip);
-	STRFREE(buddy->download);
-	STRFREE(buddy->version);
-	STRFREE(buddy->group);
-
 	close(buddy->fd);
 
+	STRFREE(buddy->download);
+	STRFREE(buddy->group);
+	STRFREE(buddy->ip);
+	STRFREE(buddy->name);
+	STRFREE(buddy->version);
+
 	free(buddy);
 }
 
@@ -676,11 +687,11 @@ void process_chat_connections(fd_set *read_set, fd_set *write_set, fd_set *exc_s
 
 void chat_socket_printf(struct chat_data *buddy, char *format, ...)
 {
-	char buf[STRING_SIZE];
+	char buf[BUFFER_SIZE];
 	va_list args;
 
 	va_start(args, format);
-	vsprintf(buf, format, args);
+	vsnprintf(buf, BUFFER_SIZE / 3, format, args);
 	va_end(args);
 
 	if (!HAS_BIT(buddy->flags, CHAT_FLAG_LINKLOST))
@@ -701,7 +712,7 @@ void chat_printf(char *format, ...)
 	va_list args;
 
 	va_start(args, format);
-	vsprintf(buf, format, args);
+	vsnprintf(buf, BUFFER_SIZE / 3, format, args);
 	va_end(args);
 
 	if (strncmp(buf, "<CHAT>", 6))
@@ -743,7 +754,7 @@ int process_chat_input(struct chat_data *buddy)
 
 	push_call("process_chat_input(%p)",buddy);
 
-	size = read(buddy->fd, buf, BUFFER_SIZE - 1000);
+	size = read(buddy->fd, buf, BUFFER_SIZE / 3);
 
 	if (size <= 0)
 	{
@@ -775,6 +786,16 @@ int process_chat_input(struct chat_data *buddy)
 
 			buddy->timeout = 0;
 
+			if (strlen(buddy->name) > 20)
+			{
+				chat_socket_printf(buddy, "%c\n%s has refused your connection because your name is too long.\n%c", CHAT_MESSAGE, gtd->chat->name, name, CHAT_END_OF_COMMAND);
+
+				chat_printf("Refusing connection from %.21s:%d, name too long. (%d characters)", buddy->ip, buddy->port, strlen(buddy->name));
+
+				pop_call();
+				return -1;
+			}
+
 			for (node = gtd->chat ; node ; node = node->next)
 			{
 				if (node != buddy && !strcasecmp(name, node->name))
@@ -819,25 +840,27 @@ int process_chat_input(struct chat_data *buddy)
 
 	if (!strncmp(buf, "YES:", 4))
 	{
-		if (sscanf(buf, "YES:%s\n", temp) == 1)
+		if ((sep = strchr(buf, '\n')) != NULL)
 		{
-			strip_vt102_codes(temp, name);
+			*sep++ = 0;
+
+			strcpy(temp, buf);
+
+			strip_vt102_codes(&temp[4], name);
 
 			RESTRING(buddy->name, name);
 
 			chat_socket_printf(buddy, "%c%s%s%c", CHAT_VERSION, "TinTin++ ", VERSION_NUM, CHAT_END_OF_COMMAND);
 
-			sep = strchr(buf, '\n');
-
-			*sep++ = 0;
-
-			get_chat_commands(buddy, sep, size - strlen(buf));
+			get_chat_commands(buddy, sep, size - strlen(temp) - 1);
 
 			pop_call();
 			return 0;
 		}
 		else
 		{
+			chat_printf("Error in processing connection negotation with %s@%s", buddy->name, buddy->ip);
+
 			pop_call();
 			return -1;
 		}
@@ -845,6 +868,8 @@ int process_chat_input(struct chat_data *buddy)
 
 	if (!strncmp(buf, "NO", 2))
 	{
+		chat_printf("Connection negotation refused by %s@%s", buddy->name, buddy->ip);
+
 		pop_call();
 		return -1;
 	}
@@ -924,12 +949,9 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 
 		*pto-- = 0;
 
-		if (strlen(txt))
+		while (isspace(*pto))
 		{
-			while (isspace(*pto))
-			{
-				*pto-- = 0;
-			}
+			*pto-- = 0;
 		}
 
 		switch (ptc)
@@ -979,11 +1001,11 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 				break;
 
 			case CHAT_VERSION:
-				if (buddy->version == NULL)
+				if (*buddy->version == 0 && *txt != 0)
 				{
 					chat_socket_printf(buddy, "%c%s%s%c", CHAT_VERSION, "TinTin++ ", VERSION_NUM, CHAT_END_OF_COMMAND);
 
-					buddy->version = strdup(txt);
+					RESTRING(buddy->version, txt);
 				}
 				break;
 
@@ -1029,11 +1051,12 @@ void get_chat_commands(struct chat_data *buddy, char *buf, int len)
 
 			case CHAT_SNOOP_DATA:
 				SET_BIT(buddy->flags, CHAT_FLAG_FORWARDBY);
+				DEL_BIT(buddy->flags, CHAT_FLAG_FORWARD);
 				chat_receive_snoop_data(buddy, txt);
 				break;
 
 			default:
-				tintin_printf(NULL, "get_chat_commands: unknown option [%d] (%s)", ptc, txt);
+				chat_printf("get_chat_commands: unknown option [%d] from %s@%s:%d (%s)", ptc, buddy->name, buddy->ip, buddy->port, txt);
 				break;
 		}
 		pti++;
@@ -1053,11 +1076,22 @@ void chat_name_change(struct chat_data *buddy, char *txt)
 
 	strip_vt102_codes(txt, name);
 
+	if (strlen(name) > 20)
+	{
+		chat_socket_printf(buddy, "%c\n%s has refused your name change because your name is too long.\n%c", CHAT_MESSAGE, gtd->chat->name, name, CHAT_END_OF_COMMAND);
+
+		chat_printf("Refusing connection from %.21s:%d, name too long. (%d characters)", buddy->ip, buddy->port, strlen(name));
+
+		close_chat(buddy, TRUE);
+
+		return;
+	}
+
 	for (node = gtd->chat ; node ; node = node->next)
 	{
 		if (node != buddy && !strcmp(name, node->name))
 		{
-			chat_socket_printf(buddy, "%c\n%s was already connected to someone named %s.\n%c", CHAT_MESSAGE, gtd->chat->name, name, CHAT_END_OF_COMMAND);
+			chat_socket_printf(buddy, "%c\n%s is already connected to someone named %s.\n%c", CHAT_MESSAGE, gtd->chat->name, name, CHAT_END_OF_COMMAND);
 
 			chat_printf("Refusing name change from %s@%s:%d, already connected to someone named %s.", buddy->name, buddy->ip, buddy->port, name);
 
@@ -1351,16 +1385,7 @@ DO_CHAT(chat_downloaddir)
 {
 	char dir[BUFFER_SIZE];
 
-	if (gtd->chat->download == NULL)
-	{
-		sprintf(dir, "%s/", getenv("HOME"));
-
-		gtd->chat->download = strdup(dir);
-
-		return;
-	}
-
-	sprintf(dir, "%s%s", left, is_suffix("/", left) ? "/" : "");
+	sprintf(dir, "%s%s", left, !str_suffix(left, "/") ? "" : "/");
 
 	RESTRING(gtd->chat->download, dir);
 
@@ -1485,6 +1510,14 @@ DO_CHAT(chat_name)
 
 		return;
 	}
+
+	if (strip_vt102_strlen(left) > 20)
+	{
+		chat_printf("Your name cannot be longer than 20 characters.");
+
+		return;
+	}
+
 	RESTRING(gtd->chat->name, left);
 
 	for (buddy = gtd->chat->next ; buddy ; buddy = buddy->next)
@@ -1774,6 +1807,7 @@ DO_CHAT(chat_zap)
 DO_CHAT(chat_accept)
 {
 	struct chat_data *buddy;
+	char path[BUFFER_SIZE];
 
 	if ((buddy = find_buddy(left)) == NULL)
 	{
@@ -1782,7 +1816,7 @@ DO_CHAT(chat_accept)
 		return;
 	}
 
-	if (buddy->file_pt == NULL)
+	if (buddy->file_name == NULL)
 	{
 		chat_printf("ERROR: You don't have a file transfer in progress with %s.", buddy->name);
 
@@ -1796,6 +1830,19 @@ DO_CHAT(chat_accept)
 		return;
 	}
 
+	sprintf(path, "%s%s", gtd->chat->download, buddy->file_name);
+
+	if ((buddy->file_pt = fopen(path, "w")) == NULL)
+	{
+		deny_file(buddy, "\nCould not create that file on receiver's end.\n");
+
+		chat_printf("ERROR: Could not create the file '%s' on your end.", buddy->file_name);
+
+		file_cleanup(buddy);
+
+		return;
+	}
+
 	buddy->file_start_time = utime();
 
 	chat_socket_printf(buddy, "%c%c", CHAT_FILE_BLOCK_REQUEST, CHAT_END_OF_COMMAND);
@@ -1991,19 +2038,18 @@ void chat_receive_file(char *arg, struct chat_data *buddy)
 
 	sprintf(path, "%s%s", gtd->chat->download, buddy->file_name);
 
-	if ((buddy->file_pt = fopen(path, "w")) == NULL)
-	{
-		deny_file(buddy, "\nCould not create that file on receiver's end.\n");
-
-		file_cleanup(buddy);
-
-		pop_call();
-		return;
-	}
-
 	chat_printf("File transfer from %s, file: %s, size: %d.", buddy->name, buddy->file_name, buddy->file_size);
 	chat_printf("Use %cchat <accept|decline> %s to proceed.", gtd->tintin_char, buddy->name);
 
+	if ((buddy->file_pt = fopen(path, "r")) != NULL)
+	{
+		chat_printf("Warning, the file already exists on your end.");
+
+		fclose(buddy->file_pt);
+
+		buddy->file_pt = NULL;
+	}
+
 	buddy->file_start_time = 0;
 
 	pop_call();
@@ -2204,7 +2250,7 @@ DO_CHAT(chat_color)
 {
 	if (*left == 0 || get_highlight_codes(gtd->ses, left, right) == FALSE)
 	{
-		chat_printf("Valid colors are:\r\n\r\nreset, bold, dim, light, dark, underscore, blink, reverse, black, red, green, yellow, blue, magenta, cyan, white, b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white");
+		chat_printf("Valid colors are:\n\nreset, bold, dim, light, dark, underscore, blink, reverse, black, red, green, yellow, blue, magenta, cyan, white, b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white");
 
 		return;
 	}
@@ -2383,7 +2429,7 @@ void chat_forward_session(struct session *ses, char *linelog)
 }
 
 /*
-	Pussy version of the ignore feature
+	The ignore feature.
 */
 
 DO_CHAT(chat_ignore)
