@@ -170,6 +170,8 @@ int connect_mud(struct session *ses, char *host, char *port)
 
 void write_line_mud(struct session *ses, char *line, int size)
 {
+	static int retry;
+
 	push_call("write_line_mud(%p,%p)",line,ses);
 
 	if (ses == gts)
@@ -190,8 +192,26 @@ void write_line_mud(struct session *ses, char *line, int size)
 
 	if (write(ses->socket, line, size) == -1)
 	{
-		syserr("write in write_line_mud");
+		if (retry++ < 10)
+		{
+			usleep(100000);
+
+			write_line_mud(ses, line, size);
+
+			pop_call();
+			return;
+		}
+		perror("write in write_line_mud");
+		cleanup_session(ses);
+
+		pop_call();
+		return;
 	}
+
+	retry = 0;
+
+	check_all_events(ses, 0, 1, "SEND OUTPUT", line);
+
 	pop_call();
 	return;
 }
@@ -273,12 +293,6 @@ void readmud(struct session *ses)
 
 				ses->more_output[0] = 0;
 			}
-			else if (*line && HAS_BIT(ses->flags, SES_FLAG_SPLIT))
-			{
-				sprintf(linebuf, "%s%s", ses->more_output, line);
-
-				ses->more_output[0] = 0;
-			}
 			else
 			{
 				strcpy(linebuf, line);
