@@ -31,87 +31,94 @@
 
 struct command_type command_table[] =
 {
-	{    "action",            do_action,            TOKEN_OPR_CMD    },
-	{    "alias",             do_alias,             TOKEN_OPR_CMD    },
-	{    "all",               do_all,               TOKEN_OPR_CMD    },
-	{    "bell",              do_bell,              TOKEN_OPR_CMD    },
-	{    "buffer",            do_buffer,            TOKEN_OPR_CMD    },
-	{    "chat",              do_chat,              TOKEN_OPR_CMD    },
-	{    "class",             do_class,             TOKEN_OPR_CMD    },
-	{    "commands",          do_commands,          TOKEN_OPR_CMD    },
-	{    "config",            do_configure,         TOKEN_OPR_CMD    },
-	{    "cr",                do_cr,                TOKEN_OPR_CMD    },
-	{    "cursor",            do_cursor,            TOKEN_OPR_CMD    },
-	{    "debug",             do_debug,             TOKEN_OPR_CMD    },
-	{    "delay",             do_delay,             TOKEN_OPR_CMD    },
-	{    "echo",              do_echo,              TOKEN_OPR_CMD    },
-	{    "end",               do_end,               TOKEN_OPR_CMD    },
-	{    "event",             do_event,             TOKEN_OPR_CMD    },
-	{    "forall",            do_forall,            TOKEN_OPR_CMD    },
-	{    "format",            do_format,            TOKEN_OPR_CMD    },
-	{    "function",          do_function,          TOKEN_OPR_CMD    },
-	{    "gag",               do_gag,               TOKEN_OPR_CMD    },
-	{    "gagline",           do_gagline,           TOKEN_OPR_CMD    },
-	{    "grep",              do_grep,              TOKEN_OPR_CMD    },
-	{    "help",              do_help,              TOKEN_OPR_CMD    },
-	{    "highlight",         do_highlight,         TOKEN_OPR_CMD    },
-	{    "history",           do_history,           TOKEN_OPR_CMD    },
-	{    "if",                do_if,                TOKEN_OPR_IF     },
-	{    "ignore",            do_ignore,            TOKEN_OPR_CMD    },
-	{    "info",              do_info,              TOKEN_OPR_CMD    },
-	{    "killall",           do_killall,           TOKEN_OPR_CMD    },
-	{    "line",              do_line,              TOKEN_OPR_CMD    },
-	{    "list",              do_list,              TOKEN_OPR_CMD    },
-	{    "log",               do_log,               TOKEN_OPR_CMD    },
-	{    "logline",           do_logline,           TOKEN_OPR_CMD    },
-	{    "loop",              do_loop,              TOKEN_OPR_CMD    },
-	{    "macro",             do_macro,             TOKEN_OPR_CMD    },
-	{    "map",               do_map,               TOKEN_OPR_CMD    },
-	{    "math",              do_math,              TOKEN_OPR_CMD    },
-	{    "message",           do_message,           TOKEN_OPR_CMD    },
-	{    "nop",               do_nop,               TOKEN_OPR_CMD    },
-	{    "parse",             do_parse,             TOKEN_OPR_CMD    },
-	{    "path",              do_path,              TOKEN_OPR_CMD    },
-	{    "pathdir",           do_pathdir,           TOKEN_OPR_CMD    },
-	{    "prompt",            do_prompt,            TOKEN_OPR_CMD    },
-	{    "read",              do_read,              TOKEN_OPR_CMD    },
-	{    "regexp",            do_regexp,            TOKEN_OPR_CMD    },
-	{    "replacestring",     do_replace,           TOKEN_OPR_CMD    },
-	{    "return",            do_return,            TOKEN_OPR_RETURN },
-	{    "run",               do_run,               TOKEN_OPR_CMD    },
-	{    "scan",              do_scan,              TOKEN_OPR_CMD    },
-	{    "script",            do_script,            TOKEN_OPR_CMD    },
-	{    "send",              do_send,              TOKEN_OPR_CMD    },
-	{    "session",           do_session,           TOKEN_OPR_CMD    },
-	{    "showme",            do_showme,            TOKEN_OPR_CMD    },
-	{    "snoop",             do_snoop,             TOKEN_OPR_CMD    },
-	{    "split",             do_split,             TOKEN_OPR_CMD    },
-	{    "substitute",        do_substitute,        TOKEN_OPR_CMD    },
-	{    "suspend",           do_suspend,           TOKEN_OPR_CMD    },
-	{    "system",            do_system,            TOKEN_OPR_CMD    },
-	{    "tab",               do_tab,               TOKEN_OPR_CMD    },
-	{    "textin",            do_textin,            TOKEN_OPR_CMD    },
-	{    "ticker",            do_tick,              TOKEN_OPR_CMD    },
-	{    "unaction",          do_unaction,          TOKEN_OPR_CMD    },
-	{    "unalias",           do_unalias,           TOKEN_OPR_CMD    },
-	{    "undelay",           do_undelay,           TOKEN_OPR_CMD    },
-	{    "unevent",           do_unevent,           TOKEN_OPR_CMD    },
-	{    "unfunction",        do_unfunction,        TOKEN_OPR_CMD    },
-	{    "ungag",             do_ungag,             TOKEN_OPR_CMD    },
-	{    "unhighlight",       do_unhighlight,       TOKEN_OPR_CMD    },
-	{    "unmacro",           do_unmacro,           TOKEN_OPR_CMD    },
-	{    "unpathdir",         do_unpathdir,         TOKEN_OPR_CMD    },
-	{    "unprompt",          do_unprompt,          TOKEN_OPR_CMD    },
-	{    "unsplit",           do_unsplit,           TOKEN_OPR_CMD    },
-	{    "unsubstitute",      do_unsubstitute,      TOKEN_OPR_CMD    },
-	{    "untab",             do_untab,             TOKEN_OPR_CMD    },
-	{    "unticker",          do_untick,            TOKEN_OPR_CMD    },
-	{    "unvariable",        do_unvariable,        TOKEN_OPR_CMD    },
-	{    "variable",          do_variable,          TOKEN_OPR_CMD    },
-	{    "while",             do_while,             TOKEN_OPR_CMD    },
-	{    "write",             do_write,             TOKEN_OPR_CMD    },
-	{    "zap",               do_zap,               TOKEN_OPR_CMD    },
-	{    "",                  NULL,                 TOKEN_OPR_CMD    }
+	{    "action",            do_action,            TOKEN_TYP_COMMAND },
+	{    "alias",             do_alias,             TOKEN_TYP_COMMAND },
+	{    "all",               do_all,               TOKEN_TYP_COMMAND },
+	{    "bell",              do_bell,              TOKEN_TYP_COMMAND },
+	{    "break",             do_nop,               TOKEN_TYP_BREAK   },
+	{    "buffer",            do_buffer,            TOKEN_TYP_COMMAND },
+	{    "case",              do_nop,               TOKEN_TYP_CASE    },
+	{    "chat",              do_chat,              TOKEN_TYP_COMMAND },
+	{    "class",             do_class,             TOKEN_TYP_COMMAND },
+	{    "commands",          do_commands,          TOKEN_TYP_COMMAND },
+	{    "config",            do_configure,         TOKEN_TYP_COMMAND },
+	{    "continue",          do_nop,               TOKEN_TYP_CONTINUE},
+	{    "cr",                do_cr,                TOKEN_TYP_COMMAND },
+	{    "cursor",            do_cursor,            TOKEN_TYP_COMMAND },
+	{    "debug",             do_debug,             TOKEN_TYP_COMMAND },
+	{    "default",           do_nop,               TOKEN_TYP_DEFAULT },
+	{    "delay",             do_delay,             TOKEN_TYP_COMMAND },
+	{    "echo",              do_echo,              TOKEN_TYP_COMMAND },
+	{    "else",              do_nop,               TOKEN_TYP_ELSE    },
+	{    "elseif",            do_nop,               TOKEN_TYP_ELSEIF  },
+	{    "end",               do_end,               TOKEN_TYP_COMMAND },
+	{    "event",             do_event,             TOKEN_TYP_COMMAND },
+	{    "forall",            do_forall,            TOKEN_TYP_COMMAND },
+	{    "foreach",           do_nop,               TOKEN_TYP_FOREACH },
+	{    "format",            do_format,            TOKEN_TYP_COMMAND },
+	{    "function",          do_function,          TOKEN_TYP_COMMAND },
+	{    "gag",               do_gag,               TOKEN_TYP_COMMAND },
+	{    "grep",              do_grep,              TOKEN_TYP_COMMAND },
+	{    "help",              do_help,              TOKEN_TYP_COMMAND },
+	{    "highlight",         do_highlight,         TOKEN_TYP_COMMAND },
+	{    "history",           do_history,           TOKEN_TYP_COMMAND },
+	{    "if",                do_nop,               TOKEN_TYP_IF      },
+	{    "ignore",            do_ignore,            TOKEN_TYP_COMMAND },
+	{    "info",              do_info,              TOKEN_TYP_COMMAND },
+	{    "killall",           do_kill,              TOKEN_TYP_COMMAND },
+	{    "line",              do_line,              TOKEN_TYP_COMMAND },
+	{    "list",              do_list,              TOKEN_TYP_COMMAND },
+	{    "log",               do_log,               TOKEN_TYP_COMMAND },
+	{    "loop",              do_nop,               TOKEN_TYP_LOOP    },
+	{    "macro",             do_macro,             TOKEN_TYP_COMMAND },
+	{    "map",               do_map,               TOKEN_TYP_COMMAND },
+	{    "math",              do_math,              TOKEN_TYP_COMMAND },
+	{    "message",           do_message,           TOKEN_TYP_COMMAND },
+	{    "nop",               do_nop,               TOKEN_TYP_COMMAND },
+	{    "parse",             do_nop,               TOKEN_TYP_PARSE   },
+	{    "path",              do_path,              TOKEN_TYP_COMMAND },
+	{    "pathdir",           do_pathdir,           TOKEN_TYP_COMMAND },
+	{    "prompt",            do_prompt,            TOKEN_TYP_COMMAND },
+	{    "read",              do_read,              TOKEN_TYP_COMMAND },
+	{    "regexp",            do_regexp,            TOKEN_TYP_COMMAND },
+	{    "replace",           do_replace,           TOKEN_TYP_COMMAND },
+	{    "return",            do_nop,               TOKEN_TYP_RETURN  },
+	{    "run",               do_run,               TOKEN_TYP_COMMAND },
+	{    "scan",              do_scan,              TOKEN_TYP_COMMAND },
+	{    "script",            do_script,            TOKEN_TYP_COMMAND },
+	{    "send",              do_send,              TOKEN_TYP_COMMAND },
+	{    "session",           do_session,           TOKEN_TYP_COMMAND },
+	{    "showme",            do_showme,            TOKEN_TYP_COMMAND },
+	{    "snoop",             do_snoop,             TOKEN_TYP_COMMAND },
+	{    "split",             do_split,             TOKEN_TYP_COMMAND },
+	{    "substitute",        do_substitute,        TOKEN_TYP_COMMAND },
+	{    "suspend",           do_suspend,           TOKEN_TYP_COMMAND },
+	{    "switch",            do_nop,               TOKEN_TYP_SWITCH  },
+	{    "system",            do_system,            TOKEN_TYP_COMMAND },
+	{    "tab",               do_tab,               TOKEN_TYP_COMMAND },
+//	{    "test",              do_test,              TOKEN_TYP_COMMAND },
+	{    "textin",            do_textin,            TOKEN_TYP_COMMAND },
+	{    "ticker",            do_tick,              TOKEN_TYP_COMMAND },
+	{    "unaction",          do_unaction,          TOKEN_TYP_COMMAND },
+	{    "unalias",           do_unalias,           TOKEN_TYP_COMMAND },
+	{    "undelay",           do_undelay,           TOKEN_TYP_COMMAND },
+	{    "unevent",           do_unevent,           TOKEN_TYP_COMMAND },
+	{    "unfunction",        do_unfunction,        TOKEN_TYP_COMMAND },
+	{    "ungag",             do_ungag,             TOKEN_TYP_COMMAND },
+	{    "unhighlight",       do_unhighlight,       TOKEN_TYP_COMMAND },
+	{    "unmacro",           do_unmacro,           TOKEN_TYP_COMMAND },
+	{    "unpathdir",         do_unpathdir,         TOKEN_TYP_COMMAND },
+	{    "unprompt",          do_unprompt,          TOKEN_TYP_COMMAND },
+	{    "unsplit",           do_unsplit,           TOKEN_TYP_COMMAND },
+	{    "unsubstitute",      do_unsubstitute,      TOKEN_TYP_COMMAND },
+	{    "untab",             do_untab,             TOKEN_TYP_COMMAND },
+	{    "unticker",          do_untick,            TOKEN_TYP_COMMAND },
+	{    "unvariable",        do_unvariable,        TOKEN_TYP_COMMAND },
+	{    "variable",          do_variable,          TOKEN_TYP_COMMAND },
+	{    "while",             do_nop,               TOKEN_TYP_WHILE   },
+	{    "write",             do_write,             TOKEN_TYP_COMMAND },
+	{    "zap",               do_zap,               TOKEN_TYP_COMMAND },
+	{    "",                  NULL,                 TOKEN_TYP_COMMAND }
 };
 
 
@@ -119,8 +126,8 @@ struct list_type list_table[LIST_MAX] =
 {
 	{    "ACTION",            "ACTIONS",            PRIORITY,    3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "ALIAS",             "ALIASES",            PRIORITY,    3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
-	{    "CLASS",             "CLASSES",            APPEND,      2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_INHERIT                                 },
-	{    "CONFIG",            "CONFIGURATIONS",     PRIORITY,    2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_INHERIT                 },
+	{    "CLASS",             "CLASSES",            ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_INHERIT                                 },
+	{    "CONFIG",            "CONFIGURATIONS",     ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_INHERIT                 },
 	{    "DELAY",             "DELAYS",             ALPHA,       3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ                                                   },
 	{    "EVENT",             "EVENTS",             ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "FUNCTION",          "FUNCTIONS",          ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
@@ -128,16 +135,14 @@ struct list_type list_table[LIST_MAX] =
 	{    "HIGHLIGHT",         "HIGHLIGHTS",         PRIORITY,    3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "HISTORY",           "HISTORIES",          APPEND,      1,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE                                                                  },
 	{    "MACRO",             "MACROS",             ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
-	{    "MAP",               "MAPPING",            APPEND,      3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE                                                                  },
-	{    "MATH",              "MATHEMATICS",        APPEND,      3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE                                                                  },
 	{    "PATH",              "PATHS",              APPEND,      2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE                                                                  },
-	{    "PATHDIR",           "PATHDIRS",           PRIORITY,    3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
+	{    "PATHDIR",           "PATHDIRS",           ALPHA,       3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "PROMPT",            "PROMPTS",            PRIORITY,    3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "SUBSTITUTE",        "SUBSTITUTIONS",      PRIORITY,    3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "TAB",               "TABS",               ALPHA,       1,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
+	{    "TABCYCLE",          "TABCYCLE",           APPEND,      1,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE                                                                  },
 	{    "TICKER",            "TICKERS",            ALPHA,       3,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
-	{    "VARIABLE",          "VARIABLES",          ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT }
-
+	{    "VARIABLE",          "VARIABLES",          ALPHA,       2,  LIST_FLAG_SHOW|LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT|LIST_FLAG_NEST }
 };
 
 
@@ -234,13 +239,6 @@ struct config_type config_table[] =
 		config_packetpatch
 	},
 
-	{
-		"REGEXP",
-		"Tintin will use regular expressions for text matching",
-		"Tintin will use the * and ? wildcards for text matching",
-		config_regexp
-	},
-
 	{
 		"REPEAT ENTER",
 		"You send the last command on an enter",
@@ -391,55 +389,247 @@ struct chat_type chat_table[] =
 
 struct array_type array_table[] =
 {
-	{     "CLR",              array_clr,           0, 0    },
-	{     "DEL",              array_del,           0, 0    },
-	{     "FND",              array_fnd,           0, 1    },
-	{     "GET",              array_get,           0, 1    },
-	{     "INS",              array_ins,           0, 1    },
-	{     "LEN",              array_len,           1, 0    },
-	{     "SET",              array_set,           0, 1    },
-	{     "SRT",              array_srt,           1, 0    },
-	{     "",                 NULL,                0, 0    }
+	{     "ADD",              array_add           },
+	{     "CLEAR",            array_clear         },
+	{     "CLR",              array_clear         },
+	{     "CREATE",           array_create        },
+	{     "DELETE",           array_delete        },
+	{     "FIND",             array_find          },
+	{     "FND",              array_find          },
+	{     "GET",              array_get           },
+	{     "INSERT",           array_insert        },
+	{     "LENGTH",           array_size          },
+	{     "SET",              array_set           },
+	{     "SIZE",             array_size          },
+	{     "SORT",             array_sort          },
+	{     "SRT",              array_sort          },
+	{     "",                 NULL                }
 };
 
 struct map_type map_table[] =
 {
-	{     "COLOR",            map_color,           1, 1    },
-	{     "CREATE",           map_create,          1, 1    },
-	{     "DELETE",           map_delete,          1, 1    },
-	{     "DESTROY",          map_destroy,         1, 1    },
-	{     "DIG",              map_dig,             1, 1    },
-	{     "EXITCMD",          map_exitcmd,         0, 1    },
-	{     "EXITDIR",          map_exitdir,         0, 1    },
-	{     "EXPLORE",          map_explore,         1, 1    },
-	{     "FIND",             map_find,            1, 1    },
-	{     "FLAG",             map_flag,            0, 1    },
-	{     "GET",              map_get,             0, 1    },
-	{     "GOTO",             map_goto,            1, 1    },
-	{     "INFO",             map_info,            1, 1    },
-	{     "INSERT",           map_insert,          0, 1    },
-	{     "JUMP",             map_jump,            0, 0    },
-	{     "LEAVE",            map_leave,           1, 1    },
-	{     "LEGENDA",          map_legend,          1, 1    },
-	{     "LINK",             map_link,            0, 1    },
-	{     "LIST",             map_list,            1, 1    },
-	{     "MAP",              map_map,             0, 0    },
-	{     "MOVE",             map_move,            1, 1    },
-	{     "NAME",             map_name,            1, 1    },
-	{     "READ",             map_read,            1, 1    },
-	{     "RETURN",           map_return,          0, 0    },
-	{     "ROOMFLAG",         map_roomflag,        0, 1    },
-	{     "RUN",              map_run,             0, 1    },
-	{     "SET",              map_set,             0, 1    },
-	{     "TRAVEL",           map_travel,          0, 1    },
-	{     "UNDO",             map_undo,            1, 1    },
-	{     "UNLINK",           map_unlink,          0, 1    },
-	{     "WRITE",            map_write,           1, 1    },
-	{     "",                 NULL,                1, 1    }
+	{     "COLOR",            map_color,           1    },
+	{     "CREATE",           map_create,          0    },
+	{     "DELETE",           map_delete,          1    },
+	{     "DESTROY",          map_destroy,         1    },
+	{     "DIG",              map_dig,             2    },
+	{     "EXITCMD",          map_exitcmd,         2    },
+	{     "EXITDIR",          map_exitdir,         2    },
+	{     "EXPLORE",          map_explore,         2    },
+	{     "FIND",             map_find,            1    },
+	{     "FLAG",             map_flag,            2    },
+	{     "GET",              map_get,             2    },
+	{     "GOTO",             map_goto,            1    },
+	{     "INFO",             map_info,            1    },
+	{     "INSERT",           map_insert,          2    },
+	{     "JUMP",             map_jump,            1    },
+	{     "LEAVE",            map_leave,           2    },
+	{     "LEGENDA",          map_legend,          1    },
+	{     "LINK",             map_link,            2    },
+	{     "LIST",             map_list,            1    },
+	{     "MAP",              map_map,             2    },
+	{     "MOVE",             map_move,            2    },
+	{     "NAME",             map_name,            2    },
+	{     "READ",             map_read,            0    },
+	{     "RETURN",           map_return,          1    },
+	{     "ROOMFLAG",         map_roomflag,        2    },
+	{     "RUN",              map_run,             2    },
+	{     "SET",              map_set,             2    },
+	{     "TRAVEL",           map_travel,          2    },
+	{     "UNDO",             map_undo,            2    },
+	{     "UNLINK",           map_unlink,          2    },
+	{     "WRITE",            map_write,           1    },
+	{     "",                 NULL,                0    }
 };
 
 struct cursor_type cursor_table[] =
 {
+	{
+		"AUTO TAB BACKWARD",
+		"Tab completion from scrollback buffer, backward",
+		"",
+		cursor_auto_tab_backward
+	},
+	{
+		"AUTO TAB FORWARD",
+		"Tab completion from scrollback buffer, forward",
+		"",
+		cursor_auto_tab_forward
+	},
+	{
+		"BACKSPACE",
+		"Delete backward character",
+		"",
+		cursor_backspace
+	},
+	{
+		"BACKWARD",
+		"Move cursor backward",
+		"",
+		cursor_left
+	},
+	{
+		"CLEAR LEFT",
+		"Delete from cursor to start of input",
+		"",
+		cursor_clear_left
+	},
+	{
+		"CLEAR LINE",
+		"Delete the input line",
+		"",
+		cursor_clear_line
+	},
+	{
+		"CLEAR RIGHT",
+		"Delete from cursor to end of input",
+		"",
+		cursor_clear_right
+	},
+	{
+		"CONVERT META",
+		"Meta convert the next character",
+		"",
+		cursor_convert_meta
+	},
+	{
+		"CTRL DELETE",
+		"Delete one character, exit on an empty line",
+		"",
+		cursor_delete_exit
+	},
+	{
+		"DELETE",
+		"Delete character at cursor",
+		"[3~",
+		cursor_delete
+	},
+	{
+		"DELETE WORD",
+		"Delete next word backward",
+		"",
+		cursor_delete_word_left
+	},
+	{
+		"ECHO ON",
+		"Turn local echoing on",
+		"",
+		cursor_echo_on
+	},
+	{
+		"ECHO OFF",
+		"Turn local echoing off",
+		"",
+		cursor_echo_off
+	},
+	{
+		"END",
+		"Move cursor to end of input",
+		"",
+		cursor_end
+	},
+	{
+		"ENTER",
+		"Process the input line",
+		"",
+		cursor_enter
+	},
+	{
+		"EXIT",
+		"Exit current session",
+		"",
+		cursor_exit
+	},
+	{
+		"FORWARD",
+		"Move cursor forward",
+		"",
+		cursor_right
+	},
+	{
+		"HISTORY NEXT",
+		"Select next command history entry",
+		"",
+		cursor_history_next
+	},
+	{
+		"HISTORY PREV",
+		"Select previous command history entry",
+		"",
+		cursor_history_prev
+	},
+	{
+		"HISTORY SEARCH",
+		"Search command history",
+		"",
+		cursor_history_search
+	},
+	{
+		"HOME",
+		"Move the cursor to start of input",
+		"",
+		cursor_home
+	},
+	{
+		"INSERT",
+		"Turn insert mode on or off",
+		"",
+		cursor_insert
+	},
+	{
+		"MIXED TAB BACKWARD",
+		"Tab completion on last word, search backward",
+		"[Z",
+		cursor_mixed_tab_backward
+	},
+	{
+		"MIXED TAB FORWARD",
+		"Tab completion on last word, search forward",
+		"\t",
+		cursor_mixed_tab_forward
+	},
+	{
+		"NEXT WORD",
+		"Move cursor to the next word",
+		"f",
+		cursor_right_word
+	},
+	{
+		"PASTE BUFFER",
+		"Paste the previously deleted input text",
+		"",
+		cursor_paste_buffer
+	},
+	{
+		"PREV WORD",
+		"Move cursor to the previous word",
+		"b",
+		cursor_left_word
+	},
+	{
+		"REDRAW INPUT",
+		"Redraw the input line",
+		"",
+		cursor_redraw_input
+	},
+	{
+		"SUSPEND",
+		"Suspend program, return with fg",
+		"",
+		cursor_suspend
+	},
+	{
+		"TAB BACKWARD",
+		"Tab completion from tab list, backward",
+		"",
+		cursor_tab_backward
+	},
+	{
+		"TAB FORWARD",
+		"Tab completion from tab list, forward",
+		"",
+		cursor_tab_forward
+	},
 	{
 		"",
 		"",
@@ -447,7 +637,6 @@ struct cursor_type cursor_table[] =
 		cursor_buffer_up
 	},
 	{
-
 		"",
 		"",
 		"[6~",
@@ -471,12 +660,6 @@ struct cursor_type cursor_table[] =
 		"",
 		cursor_buffer_lock
 	},
-	{
-		"HOME",
-		"Move the cursor to start of input",
-		"",
-		cursor_home
-	},
 	{
 		"",
 		"",
@@ -495,38 +678,13 @@ struct cursor_type cursor_table[] =
 		"OH",
 		cursor_home
 	},
-
-	{
-		"BACKWARD",
-		"Move cursor backward",
-		"",
-		cursor_left
-	},
 	{
 		"",
 		"",
 		"OD",
 		cursor_left
 	},
-	{
-		"CLEAR LINE",
-		"Delete the input line",
-		"",
-		cursor_clear_line
-	},
-	{
-		"CTRL DELETE",
-		"Delete character at cursor, or exit on empty line",
-		"",
-		cursor_delete_exit
-	},
 
-	{
-		"END",
-		"Move cursor to end of input",
-		"",
-		cursor_end
-	},
 	{
 		"",
 		"",
@@ -545,102 +703,30 @@ struct cursor_type cursor_table[] =
 		"OF",
 		cursor_end
 	},
-	{
-		"FORWARD",
-		"Move cursor forward",
-		"",
-		cursor_right
-	},
 	{
 		"",
 		"",
 		"OC",
 		cursor_right
 	},
-	{
-		"BACKSPACE",
-		"Delete backward character",
-		"",
-		cursor_backspace
-	},
 	{
 		"",
 		"",
 		"",
 		cursor_backspace
 	},
-	{
-		"TAB",
-		"Tab completion on last word",
-		"\t",
-		cursor_tab
-	},
-	{
-		"ENTER",
-		"Process the input line",
-		"",
-		cursor_enter
-	},
-	{
-		"CLEAR RIGHT",
-		"Delete from cursor to end of input",
-		"",
-		cursor_clear_right
-	},
-	{
-		"REDRAW INPUT",
-		"Redraw the input line",
-		"",
-		cursor_redraw_input
-	},
-	{
-		"HISTORY NEXT",
-		"Select next command history entry",
-		"",
-		cursor_history_next
-	},
 	{
 		"",
 		"",
 		"OB",
 		cursor_history_next
 	},
-	{
-		"HISTORY PREV",
-		"Select previous command history entry",
-		"",
-		cursor_history_prev
-	},
 	{
 		"",
 		"",
 		"OA",
 		cursor_history_prev
 	},
-	{
-		"HISTORY SEARCH",
-		"Search command history",
-		"",
-		cursor_history_search
-	},
-	{
-		"CLEAR LEFT",
-		"Delete from cursor to start of input",
-		"",
-		cursor_clear_left
-	},
-	{
-		"CONVERT META",
-		"Meta convert the next character",
-		"",
-		cursor_convert_meta
-	},
-	{
-		"DELETE WORD",
-		"Delete next word backward",
-		"",
-		cursor_delete_word_left
-	},
 	{
 		"",
 		"",
@@ -653,60 +739,6 @@ struct cursor_type cursor_table[] =
 		"d",
 		cursor_delete_word_right
 	},
-	{
-		"PASTE BUFFER",
-		"Paste the previously deleted input text",
-		"",
-		cursor_paste_buffer
-	},
-	{
-		"SUSPEND",
-		"Suspend program, return with fg",
-		"",
-		cursor_suspend
-	},
-	{
-		"AUTO TAB",
-		"Tab completion from scrollback buffer",
-		"",
-		cursor_auto_tab
-	},
-	{
-		"DELETE",
-		"Delete character at cursor",
-		"[3~",
-		cursor_delete
-	},
-	{
-		"EXIT",
-		"Exit current session",
-		"",
-		cursor_exit
-	},
-	{
-		"ECHO",
-		"Turn local echoing on or off",
-		"",
-		cursor_echo
-	},
-	{
-		"INSERT",
-		"Turn insert mode on or off",
-		"",
-		cursor_insert
-	},
-	{
-		"NEXT WORD",
-		"Move cursor to the next word",
-		"f",
-		cursor_right_word
-	},
-	{
-		"PREV WORD",
-		"Move cursor to the previous word",
-		"b",
-		cursor_left_word
-	},
 	{
 		"",
 		"",
@@ -731,15 +763,17 @@ struct timer_type timer_table[] =
 
 struct event_type event_table[] =
 {
-	{    "END OF PATH",                            "Triggers when walking the last room."  },
-	{    "IAC ",                                   "Triggers on telopt negotiation."       },
-	{    "MAP ENTER ROOM",                         "Triggers when entering a map room."    },
-	{    "PROGRAM TERMINATION",                    "Triggers when main session exists."    },
-	{    "RECEIVED INPUT",                         "Triggers when new input is received."  },
-	{    "RECEIVED LINE",                          "Triggers when a new line is received." },
-	{    "SESSION CONNECTED",                      "Triggers when a new session connects." },
-	{    "SESSION DISCONNECTED",                   "Triggers when a session disconnects."  },
-	{    "",                                       ""                                      }
+	{    "END OF PATH",                            "Triggers when walking the last room."    },
+	{    "IAC ",                                   "Triggers on telopt negotiation."         },
+	{    "MAP ENTER ROOM",                         "Triggers when entering a map room."      },
+	{    "MAP EXIT ROOM",                          "Triggers when exiting a map room."       },
+	{    "PROGRAM TERMINATION",                    "Triggers when main session exists."      },
+	{    "RECEIVED INPUT",                         "Triggers when new input is received."    },
+	{    "RECEIVED LINE",                          "Triggers when a new line is received."   },
+	{    "SEND OUTPUT",                            "Triggers when sending output."           },
+	{    "SESSION CONNECTED",                      "Triggers when a new session connects."   },
+	{    "SESSION DISCONNECTED",                   "Triggers when a session disconnects."    },
+	{    "",                                       ""                                        }
 };
 
 struct path_type path_table[] =
@@ -761,7 +795,7 @@ struct line_type line_table[] =
 {
 	{    "GAG",               line_gag               },
 	{    "LOG",               line_log               },
-	{    "LOGRAW",            line_lograw            },
+	{    "LOGVERBATIM",       line_logverbatim       },
 	{    "",                  NULL                   }
 };
 
