@@ -26,115 +26,45 @@
 *                     recoded by Igor van den Hoven 2004                      *
 ******************************************************************************/
 
-#include <regex.h>
 #include <sys/types.h>
 #include <pcre.h>
 
 #include "tintin.h"
 
-/******************************************************************************
-* Simple regexp that skips vt102 codes and supports the * and ? wildcards     *
-******************************************************************************/
 
 int match(struct session *ses, char *str, char *exp)
 {
-	char expbuf[BUFFER_SIZE], tmpbuf[BUFFER_SIZE];
+	char expbuf[BUFFER_SIZE];
 
-	if (HAS_BIT(ses->flags, SES_FLAG_REGEXP))
+	if (ses)
 	{
-		substitute(ses, exp, tmpbuf, SUB_VAR|SUB_FUN|SUB_ESC);
-
-		sprintf(expbuf, "^%s$", tmpbuf);
-
-		return regexp(str, expbuf);
+		sprintf(expbuf, "^%s$", exp);
+		
+		substitute(ses, expbuf, expbuf, SUB_VAR|SUB_FUN);
 	}
 	else
 	{
-		substitute(ses, exp, expbuf, SUB_VAR|SUB_FUN);
-
-		return glob(str, expbuf);
+		sprintf(expbuf, "^%s$", exp);
 	}
+	return tintin_regexp(NULL, str, expbuf, 0, 0);
 }
 
 int find(struct session *ses, char *str, char *exp)
 {
-	char expbuf[BUFFER_SIZE], tmpbuf[BUFFER_SIZE];
+	char expbuf[BUFFER_SIZE];
 
-	if (HAS_BIT(ses->flags, SES_FLAG_REGEXP))
+	if (ses)
 	{
-		strip_vt102_codes(str, tmpbuf);
+		substitute(ses, exp, expbuf, SUB_VAR|SUB_FUN);
 
-		substitute(ses, exp, expbuf, SUB_VAR|SUB_FUN|SUB_ESC);
-
-		return regexp(tmpbuf, expbuf);
+		return tintin_regexp(NULL, str, expbuf, 0, 0);
 	}
 	else
 	{
-		substitute(ses, exp, tmpbuf, SUB_VAR|SUB_FUN);
-
-		sprintf(expbuf, "*%s*", tmpbuf);
-
-		return glob(str, expbuf);
+		return tintin_regexp(NULL, str, exp, 0, 0);
 	}
 }
 
-int glob(char *str, char *exp)
-{
-	short cnt;
-
-	while (*exp)
-	{
-		while ((cnt = skip_vt102_codes(str)) != 0)
-		{
-			str += cnt;
-		}
-
-#ifdef BIG5
-		if (*exp & 0x80 && *str & 0x80)
-		{
-			if (*++exp != *++str)
-			{
-				return FALSE;
-			}
-			continue;
-		}
-#endif
-
-		switch (*exp)
-		{
-			case '?':
-				if (*str == 0)
-				{
-					return FALSE;
-				}
-				exp++;
-				str++;
-				break;
-
-			case '*':
-				for (cnt = strlen(str) ; cnt >= 0 ; cnt--)
-				{
-					if (glob(&str[cnt], exp + 1))
-					{
-						return TRUE;
-					}
-				}
-				return FALSE;
-
-			case '\\':
-				exp++;
-
-			default:
-				if (*exp++ != *str++)
-				{
-					return FALSE;
-				}
-				break;
-		}
-	}
-	return (*exp == *str);
-}
-
 DO_COMMAND(do_regexp)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE], is_true[BUFFER_SIZE], is_false[BUFFER_SIZE];
@@ -144,7 +74,7 @@ DO_COMMAND(do_regexp)
 	arg = get_arg_in_braces(arg, is_true,  1);
 	arg = get_arg_in_braces(arg, is_false, 1);
 
-	if (*right == 0 || *is_true == 0)
+	if (*is_true == 0)
 	{
 		tintin_printf2(ses, "SYNTAX: #REGEXP {string} {expression} {true} {false}.");
 	}
@@ -153,7 +83,7 @@ DO_COMMAND(do_regexp)
 		substitute(ses, left,  left,  SUB_VAR|SUB_FUN);
 		substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
-		if (regexp_compare(NULL, left, right, 0, SUB_CMD))
+		if (tintin_regexp(NULL, left, right, 0, SUB_CMD))
 		{
 			substitute(ses, is_true, is_true, SUB_CMD);
 
@@ -207,7 +137,23 @@ int regexp_compare(pcre *nodepcre, char *str, char *exp, int option, int flag)
 			}
 			break;
 
+		case SUB_CMD + SUB_FIX:
+			for (i = 0 ; i < matches ; i++)
+			{
+				j = gtd->args[i];
+
+				gtd->cmds[j] = refstring(gtd->cmds[j], "%.*s", match[i*2+1] - match[i*2], &str[match[i*2]]);
+			}
+			break;
+
 		case SUB_ARG:
+			for (i = 0 ; i < matches ; i++)
+			{
+				gtd->vars[i] = refstring(gtd->vars[i], "%.*s", match[i*2+1] - match[i*2], &str[match[i*2]]);
+			}
+			break;
+
+		case SUB_ARG + SUB_FIX:
 			for (i = 0 ; i < matches ; i++)
 			{
 				j = gtd->args[i];
@@ -233,10 +179,6 @@ pcre *regexp_compile(char *exp, int option)
 	return pcre_compile(exp, option, &error, &i, NULL);
 }
 
-int regexp(char *str, char *exp)
-{
-	return regexp_compare(NULL, str, exp, 0, 0);
-}
 
 /******************************************************************************
 * copy *string into *result, but substitute the various expressions with the  *
@@ -255,7 +197,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 	pti = string;
 	pto = (string == result) ? buffer : result;
 
-	DEL_BIT(flags_neol, SUB_EOL);
+	DEL_BIT(flags_neol, SUB_EOL|SUB_LNF);
 
 	while (TRUE)
 	{
@@ -309,85 +251,56 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 
 			case '$':
-				if (!VERBATIM && HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '$'))
+				if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '$'))
 				{
+					int def = FALSE;
+
 					if (pti[1] == '$')
 					{
 						while (pti[1] == '$')
 						{
 							*pto++ = *pti++;
 						}
-						pti++;
 
-						continue;
-					}
-
-					if (pti[1] == DEFAULT_OPEN)
-					{
-						for (ptt = buf, i = 2 ; pti[i] && pti[i] != DEFAULT_CLOSE ; i++)
+						if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]))
 						{
-							*ptt++ = pti[i];
-						}
-
-						if (pti[i] == DEFAULT_CLOSE)
-						{
-							*ptt = 0;
-							substitute(ses, buf, temp, flags_neol);
+							pti++;
 						}
 						else
 						{
-							strcpy(temp, "");
+							*pto++ = *pti++;
 						}
-						i++;
+						continue;
 					}
 
-					if (isalnum(pti[1]) || pti[1] == '_')
+					pti++;
+
+					if (*pti == DEFAULT_OPEN)
 					{
-						for (ptt = temp, i = 1 ; isalnum(pti[i]) || pti[i] == '_' ; i++)
+						def = TRUE;
+
+						pti = get_arg_in_braces(pti, buf, TRUE);
+
+						substitute(ses, buf, temp, flags_neol);
+					}
+					else
+					{
+						ptt = temp;
+
+						while (isalnum(*pti) || *pti == '_')
 						{
-							*ptt++ = pti[i];
+							*ptt++ = *pti++;
 						}
 						*ptt = 0;
 					}
 
-					if ((node = searchnode_list(ses->list[LIST_VARIABLE], temp)) != NULL)
-					{
-						substitute(ses, node->right, pto, flags_neol - SUB_VAR);
+					pti = get_arg_at_brackets(pti, temp + strlen(temp));
 
-						if (pti[1] == DEFAULT_OPEN)
-						{
-							pti += 3 + strlen(buf);
-						}
-						else
-						{
-							pti += 1 + strlen(temp);
-						}
+					get_nest_node(ses->list[LIST_VARIABLE], temp, buf, def);
 
-						if (*pti == '[')
-						{
-							for (ptt = temp, i = 1 ; pti[i] && pti[i] != ']' ; i++)
-							{
-								*ptt++ = pti[i];
-							}
-							*ptt = 0;
+					substitute(ses, buf, pto, flags_neol - SUB_VAR);
 
-							if (pti[i] == ']')
-							{
-								pti += 2 + strlen(temp);
-
-								substitute(ses, temp, temp, SUB_VAR|SUB_FUN);
-
-								get_list_item(ses, node, temp);
-
-								substitute(ses, temp, pto, flags_neol - SUB_VAR);
-							}
-						}
-						pto += strlen(pto);
-					}
-					else
-					{
-						*pto++ = *pti++;
-					}
+					pto += strlen(pto);
 				}
 				else
 				{
@@ -519,7 +432,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			case '@':
-				if (!VERBATIM && HAS_BIT(flags, SUB_FUN))
+				if (HAS_BIT(flags, SUB_FUN))
 				{
 					for (ptt = temp, i = 1 ; isalnum(pti[i]) || pti[i] == '_' ; i++)
 					{
@@ -527,7 +440,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					}
 					*ptt = 0;
 
-					node = searchnode_list(ses->list[LIST_FUNCTION], temp);
+					node = search_node_list(ses->list[LIST_FUNCTION], temp);
 
 					if (node == NULL || pti[i] != DEFAULT_OPEN)
 					{
@@ -538,7 +451,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					pti = get_arg_in_braces(&pti[i], temp, FALSE);
 
-					substitute(ses, temp, buf, SUB_VAR|SUB_FUN);
+					substitute(ses, temp, buf, flags_neol);
 
 					show_debug(ses, LIST_FUNCTION, "#DEBUG FUNCTION {%s}", node->left);
 
@@ -679,6 +592,56 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						pti += isdigit(pti[2]) ? 3 : 2;
 					}
 				}
+				else if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '&'))
+				{
+					int def = 0;
+
+					if (pti[1] == '&')
+					{
+						while (pti[1] == '&')
+						{
+							*pto++ = *pti++;
+						}
+
+						if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]))
+						{
+							pti++;
+						}
+						else
+						{
+							*pto++ = *pti++;
+						}
+						continue;
+					}
+
+
+					pti++;
+
+					if (*pti == DEFAULT_OPEN)
+					{
+						def = TRUE;
+
+						pti = get_arg_in_braces(pti, buf, TRUE);
+
+						substitute(ses, buf, temp, flags_neol);
+					}
+					else
+					{
+						for (ptt = temp ; isalnum(*pti) || *pti == '_' ; i++)
+						{
+							*ptt++ = *pti++;
+						}
+						*ptt = 0;
+					}
+
+					pti = get_arg_at_brackets(pti, temp + strlen(temp));
+
+					get_nest_index(ses->list[LIST_VARIABLE], temp, buf, def);
+
+					substitute(ses, buf, pto, flags_neol - SUB_VAR);
+
+					pto += strlen(pto);
+				}
 				else
 				{
 					*pto++ = *pti++;
@@ -686,7 +649,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			case '\\':
-				if (!VERBATIM && HAS_BIT(flags, SUB_ESC))
+				if (HAS_BIT(flags, SUB_ESC))
 				{
 					pti++;
 
@@ -789,26 +752,92 @@ int check_one_regexp(struct session *ses, struct listnode *node, char *line, cha
 		str = line;
 	}	
 
-	return tintin_regexp(node->regex, str, exp, option);
+	return tintin_regexp(node->regex, str, exp, option, SUB_ARG);
 }
 
 /*
-	Keep synched with tintin_regexp_compile
+	Keep synched with tintin_regexp and tintin_regexp_compile
 */
 
-int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
+int tintin_regexp_check(char *exp)
+{
+	if (*exp == '^')
+	{
+		return TRUE;
+	}
+
+	while (*exp)
+	{
+#ifdef BIG5
+		if (*exp & 0x80)
+		{
+			exp += 2;
+			continue;
+		}
+#endif		
+		switch (exp[0])
+		{
+			case '\\':
+			case '{':
+				return TRUE;
+
+			case '$':
+				if (exp[1] == 0)
+				{
+					return TRUE;
+				}
+				break;
+
+			case '%':
+				switch (exp[1])
+				{
+					case '0':
+					case '1':
+					case '2':
+					case '3':
+					case '4':
+					case '5':
+					case '6':
+					case '7':
+					case '8':
+					case '9':
+					case 'd':
+					case 'D':
+					case 'i':
+					case 'I':
+					case 's':
+					case 'S':
+					case 'w':
+					case 'W':
+					case '?':
+					case '*':
+					case '+':
+					case '.':
+					case '%':
+						return TRUE;
+				}
+				break;
+		}
+		exp++;
+	}
+	return FALSE;
+}
+				
+int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option, int flag)
 {
 	char out[BUFFER_SIZE], *pti, *pto;
-	int arg, var;
-
-	arg = var = 1;
+	int arg = 1, var = 1, fix = 0;
 
 	pti = exp;
 	pto = out;
 
+	while (*pti == '^')
+	{
+		*pto++ = *pti++;
+	}
+
 	while (*pti)
 	{
-
 #ifdef BIG5
 		if (*pti & 0x80)
 		{
@@ -841,10 +870,29 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 			case '?':
 			case '+':
 			case '*':
+			case '^':
 				*pto++ = '\\';
 				*pto++ = *pti++;
 				break;
 
+			case '$':
+				if (pti[1] != DEFAULT_OPEN && !isalnum(pti[1]))
+				{
+					int i = 0;
+
+					while (pti[++i] == '$')
+					{
+						continue;
+					}
+
+					if (pti[i])
+					{
+						*pto++ = '\\';
+					}
+				}
+				*pto++ = *pti++;
+				break;
+
 			case '%':
 				switch (pti[1])
 				{
@@ -858,6 +906,7 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 					case '7':
 					case '8':
 					case '9':
+						fix = SUB_FIX;
 						arg = isdigit(pti[2]) ? (pti[1] - '0') * 10 + (pti[2] - '0') : pti[1] - '0';
 						gtd->args[up(var)] = up(arg);
 						pti += isdigit(pti[2]) ? 3 : 2;
@@ -865,20 +914,6 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 						pto += strlen(pto);
 						break;
 
-					case 'w':
-						gtd->args[up(var)] = up(arg);
-						pti += 2;
-						strcpy(pto, *pti == 0 ? "([a-zA-Z]*)" : "([a-zA-Z]*?)");
-						pto += strlen(pto);
-						break;
-
-					case 'W':
-						gtd->args[up(var)] = up(arg);
-						pti += 2;
-						strcpy(pto, *pti == 0 ? "([^a-zA-Z]*)" : "([^a-zA-Z]*?)");
-						pto += strlen(pto);
-						break;
-
 					case 'd':
 						gtd->args[up(var)] = up(arg);
 						pti += 2;
@@ -893,6 +928,18 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 						pto += strlen(pto);
 						break;
 
+					case 'i':
+						pti += 2;
+						strcpy(pto, "(?i)");
+						pto += strlen(pto);
+						break;
+
+					case 'I':
+						pti += 2;
+						strcpy(pto, "(?-i)");
+						pto += strlen(pto);
+						break;
+
 					case 's':
 						gtd->args[up(var)] = up(arg);
 						pti += 2;
@@ -907,6 +954,20 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 						pto += strlen(pto);
 						break;
 
+					case 'w':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([a-zA-Z]*)" : "([a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'W':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([^a-zA-Z]*)" : "([^a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
+
 					case '?':
 						gtd->args[up(var)] = up(arg);
 						pti += 2;
@@ -953,7 +1014,7 @@ int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 	}
 	*pto = 0;
 
-	return regexp_compare(nodepcre, str, out, option, SUB_ARG);
+	return regexp_compare(nodepcre, str, out, option, flag + fix);
 }
 
 pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
@@ -969,6 +1030,11 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 		SET_BIT(node->flags, NODE_FLAG_META);
 	}
 
+	while (*pti == '^')
+	{
+		*pto++ = *pti++;
+	}
+
 	while (*pti)
 	{
 
@@ -1004,15 +1070,43 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 				*pto++ = ')';
 				break;
 
-			case '$':
-			case '@':
-				if (pti[1])
+			case '&':
+				if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '&')
 				{
 					return NULL;
 				}
 				*pto++ = *pti++;
 				break;
 
+			case '@':
+				if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '@')
+				{
+					return NULL;
+				}
+				*pto++ = *pti++;
+				break;
+
+			case '$':
+				if (pti[1] == DEFAULT_OPEN || isalnum(pti[1]))
+				{
+					return NULL;
+				}
+				{
+					int i = 0;
+
+					while (pti[++i] == '$')
+					{
+						continue;
+					}
+
+					if (pti[i])
+					{
+						*pto++ = '\\';
+					}
+				}
+				*pto++ = *pti++;
+				break;
+
 			case '[':
 			case ']':
 			case '(':
@@ -1022,6 +1116,7 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 			case '?':
 			case '+':
 			case '*':
+			case '^':
 				*pto++ = '\\';
 				*pto++ = *pti++;
 				break;
@@ -1044,18 +1139,6 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 						pto += strlen(pto);
 						break;
 
-					case 'w':
-						pti += 2;
-						strcpy(pto, *pti == 0 ? "([a-zA-Z]*)" : "([a-zA-Z]*?)");
-						pto += strlen(pto);
-						break;
-
-					case 'W':
-						pti += 2;
-						strcpy(pto, *pti == 0 ? "([^a-zA-Z]*)" : "([^a-zA-Z]*?)");
-						pto += strlen(pto);
-						break;
-
 					case 'd':
 						pti += 2;
 						strcpy(pto, *pti == 0 ? "([0-9]*)" : "([0-9]*?)");
@@ -1068,6 +1151,18 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 						pto += strlen(pto);
 						break;
 
+					case 'i':
+						pti += 2;
+						strcpy(pto, "(?i)");
+						pto += strlen(pto);
+						break;
+
+					case 'I':
+						pti += 2;
+						strcpy(pto, "(?-i)");
+						pto += strlen(pto);
+						break;
+
 					case 's':
 						pti += 2;
 						strcpy(pto, *pti == 0 ? "(\\s*)" : "(\\s*?)");
@@ -1080,6 +1175,18 @@ pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
 						pto += strlen(pto);
 						break;
 
+					case 'w':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([a-zA-Z]*)" : "([a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'W':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([^a-zA-Z]*)" : "([^a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
+
 					case '?':
 						pti += 2;
 						strcpy(pto, *pti == 0 ? "(.?)" : "(.??)");
