@@ -151,20 +151,25 @@ void poll_input(void)
 	fd_set readfds;
 	static struct timeval to;
 
-	FD_ZERO(&readfds);
-
-	FD_SET(0, &readfds);
-
-	if (select(FD_SETSIZE, &readfds, NULL, NULL, &to) <= 0)
+	while (TRUE)
 	{
-		return;
-	}
+		FD_ZERO(&readfds);
 
-	if (FD_ISSET(0, &readfds))
-	{
-		process_input();
+		FD_SET(0, &readfds);
 
-		poll_input();
+		if (select(FD_SETSIZE, &readfds, NULL, NULL, &to) <= 0)
+		{
+			return;
+		}
+
+		if (FD_ISSET(0, &readfds))
+		{
+			process_input();
+		}
+		else
+		{
+			return;
+		}
 	}
 }
 
@@ -291,9 +296,9 @@ void tick_update(void)
 
 		root = ses->list[LIST_TICKER];
 
-		for (node = root->f_node ; node ; node = root->update)
+		for (root->update = 0 ; root->update < root->used ; root->update++)
 		{
-			root->update = node->next;
+			node = root->list[root->update];
 
 			if (node->data == 0)
 			{
@@ -327,9 +332,9 @@ void delay_update(void)
 
 		root = ses->list[LIST_DELAY];	
 
-		for (node = root->f_node ; node ; node = root->update)
+		for (root->update = 0 ; root->update < root->used ; root->update++)
 		{
-			root->update = node->next;
+			node = root->list[root->update];
 
 			if (node->data == 0)
 			{
@@ -342,7 +347,7 @@ void delay_update(void)
 
 				script_driver(ses, LIST_DELAY, node->right);
 
-				deletenode_list(ses, node, LIST_DELAY);
+				delete_node_list(ses, LIST_DELAY, node);
 			}
 		}
 	}
